<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 114 $
 * $Id: ClientFactory.class.php 114 2006-09-15 23:47:37Z ipso $
 * $Date: 2006-09-15 16:47:37 -0700 (Fri, 15 Sep 2006) $
 */
class ClientFactory extends Factory {
	protected $table = 'client';
	protected $pk_sequence_name = 'client_id_seq'; //PK Sequence name

	protected $company_obj = NULL;

	protected $status_options = array(
										10 => 'ACTIVE',
										20 => 'HOLD',
										30 => 'CANCELLED'
									);

	protected $company_validator_msg = 'Company is invalid';
	protected $company_validator_label = 'company';

	protected $status_validator_options_msg = 'Incorrect Status';
	protected $status_validator_label = 'status';

	protected $group_validator_msg = 'Group is invalid';
	protected $group_validator_label = 'group';

	protected $companyname_validator_length_msg = 'Company Name is too short or too long';
	protected $companyname_validator_label = 'company_name';

	protected $website_validator_length_msg = 'Website is too short or too long';
	protected $website_validator_label = 'website';

	protected $user_validator_msg = 'Contact User is invalid';
	protected $salescontact_validator_label = 'sales_contact';
	protected $supportcontact_validator_label = 'support_contact';

	protected $otherid1_validator_islength_msg = 'Other ID 1 is invalid';
	protected $otherid1_validator_label = 'other_id1';

	protected $otherid2_validator_islength_msg = 'Other ID 2 is invalid';
	protected $otherid2_validator_label = 'other_id2';

	protected $otherid3_validator_islength_msg = 'Other ID 3 is invalid';
	protected $otherid3_validator_label = 'other_id3';

	protected $otherid4_validator_islength_msg = 'Other ID 4 is invalid';
	protected $otherid4_validator_label = 'other_id4';

	protected $otherid5_validator_islength_msg = 'Other ID 5 is invalid';
	protected $otherid5_validator_label = 'other_id5';

	protected $note_validator_length_msg = 'Note is too short or too long';
	protected $note_validator_label = 'note';

	function getCompanyObject() {
		if ( is_object($this->company_obj) ) {
			return $this->company_obj;
		} else {
			$clf = new CompanyListFactory();
			$clf->getById( $this->getCompany() );
			if ( $clf->getRecordCount() > 0 ) {
				$this->company_obj = $clf->getCurrent();
				return $this->company_obj;
			}

			return FALSE;
		}
	}

	function getCompany() {
		if ( isset($this->data['company_id']) ) {
			return $this->data['company_id'];
		}

		return FALSE;
	}
	function setCompany($id) {
		$id = trim($id);

		Debug::Text('Company ID: '. $id, __FILE__, __LINE__, __METHOD__,10);
		$clf = new CompanyListFactory();

		if ( $this->Validator->isResultSetWithRows(	$this->company_validator_label,
													$clf->getByID($id),
													$this->company_validator_msg
													) ) {

			$this->data['company_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getStatus() {
		//Have to return the KEY because it should always be a drop down box.
		//return Option::getByKey($this->data['status_id'], $this->getOptions('status') );
		return $this->data['status_id'];
	}
	function setStatus($status) {
		$status = trim($status);

		$key = Option::getByValue($status, $this->getOptions('status') );
		if ($key !== FALSE) {
			$status = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->status_validator_label,
											$status,
											$this->status_validator_options_msg,
											$this->getOptions('status')) ) {

			$this->data['status_id'] = $status;

			return FALSE;
		}

		return FALSE;
	}

	function getGroup() {
		if ( isset($this->data['group_id']) ) {
			return $this->data['group_id'];
		}

		return FALSE;
	}
	function setGroup($id) {
		$id = trim($id);

		Debug::Text('Group ID: '. $id, __FILE__, __LINE__, __METHOD__,10);
		$cglf = new ClientGroupListFactory();

		if (	$id == 0
				OR
				$this->Validator->isResultSetWithRows(	$this->group_validator_label,
													$cglf->getByID($id),
													$this->group_validator_msg
													) ) {

			$this->data['group_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getCompanyName() {
		if ( isset($this->data['company_name']) ) {
			return $this->data['company_name'];
		}
	}
	function setCompanyName($name) {
		$name = trim($name);

		if 	(	$this->Validator->isLength(		$this->companyname_validator_label,
												$name,
												$this->companyname_validator_length_msg,
												2,
												100) ) {

			$this->data['company_name'] = $name;

			return TRUE;
		}

		return FALSE;
	}

	function getWebsite() {
		if ( isset($this->data['website']) ) {
			return $this->data['website'];
		}
	}
	function setWebsite($val) {
		$val = trim($val);

		if 	(	$val == ''
				OR
				$this->Validator->isLength(		$this->website_validator_label,
												$val,
												$this->website_validator_length_msg,
												0,
												250) ) {

			$this->data['website'] = $val;

			return TRUE;
		}

		return FALSE;
	}

	function getNote() {
		if ( isset($this->data['note']) ) {
			return $this->data['note'];
		}
	}
	function setNote($val) {
		$val = trim($val);

		if 	(	$val == ''
				OR
				$this->Validator->isLength(		$this->note_validator_label,
												$val,
												$this->note_validator_length_msg,
												0,
												1024) ) {

			$this->data['note'] = $val;

			return TRUE;
		}

		return FALSE;
	}

	function getSalesContact() {
		if ( isset($this->data['sales_contact']) ) {
			return $this->data['sales_contact'];
		}

		return FALSE;
	}
	function setSalesContact($id) {
		$id = trim($id);

		$ulf = new UserListFactory();

		if ( !empty($id)
				AND 	$this->Validator->isResultSetWithRows(	$this->salescontact_validator_label,
																$ulf->getByID($id),
																$this->user_validator_msg
																) ) {

			$this->data['sales_contact'] = $id;

			return TRUE;
		}

		return FALSE;

	}

	function getSupportContact() {
		if ( isset($this->data['support_contact']) ) {
			return $this->data['support_contact'];
		}

		return FALSE;
	}
	function setSupportContact($id) {
		$id = trim($id);

		$ulf = new UserListFactory();

		if ( !empty($id)
				AND $this->Validator->isResultSetWithRows(	$this->supportcontact_validator_label,
															$ulf->getByID($id),
															$this->user_validator_msg
															) ) {

			$this->data['support_contact'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getDefaultContact( $type_id ) {
		if ($type_id == '' ) {
			return FALSE;
		}

		$cclf = new ClientContactListFactory();
		$cclf->getByTypeIdAndDefaultAndClientIdAndCompanyId( $type_id, TRUE, $this->getId(), $this->getCompany() );
		if ( $cclf->getRecordCount() == 1 ) {
			$cc_obj = $cclf->getCurrent();

			return $cc_obj->getId();
		}

		return FALSE;
	}

	function getDefaultBillingContact() {
		return $this->getDefaultContact( 10 );
	}

	function getDefaultShippingContact() {
		return $this->getDefaultContact( 20 );
	}

	function getDefaultOtherContact() {
		return $this->getDefaultContact( 30 );
	}

	function getOtherID1() {
		if ( isset($this->data['other_id1']) ) {
			return $this->data['other_id1'];
		}

		return FALSE;
	}
	function setOtherID1($value) {
		$value = trim($value);

		if (	$value == ''
				OR
				$this->Validator->isLength(	$this->otherid1_validator_label,
											$value,
											$this->otherid1_validator_islength_msg,
											1,255) ) {

			$this->data['other_id1'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getOtherID2() {
		if ( isset($this->data['other_id2']) ) {
			return $this->data['other_id2'];
		}

		return FALSE;
	}
	function setOtherID2($value) {
		$value = trim($value);

		if (	$value == ''
				OR
				$this->Validator->isLength(	$this->otherid2_validator_label,
											$value,
											$this->otherid2_validator_islength_msg,
											1,255) ) {

			$this->data['other_id2'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getOtherID3() {
		if ( isset($this->data['other_id3']) ) {
			return $this->data['other_id3'];
		}

		return FALSE;
	}
	function setOtherID3($value) {
		$value = trim($value);

		if (	$value == ''
				OR
				$this->Validator->isLength(	$this->otherid3_validator_label,
											$value,
											$this->otherid3_validator_islength_msg,
											1,255) ) {

			$this->data['other_id3'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getOtherID4() {
		if ( isset($this->data['other_id4']) ) {
			return $this->data['other_id4'];
		}

		return FALSE;
	}
	function setOtherID4($value) {
		$value = trim($value);

		if (	$value == ''
				OR
				$this->Validator->isLength(	$this->otherid4_validator_label,
											$value,
											$this->otherid4_validator_islength_msg,
											1,255) ) {

			$this->data['other_id4'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getOtherID5() {
		if ( isset($this->data['other_id5']) ) {
			return $this->data['other_id5'];
		}

		return FALSE;
	}
	function setOtherID5($value) {
		$value = trim($value);

		if (	$value == ''
				OR
				$this->Validator->isLength(	$this->otherid5_validator_label,
											$value,
											$this->otherid5_validator_islength_msg,
											1,255) ) {

			$this->data['other_id5'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function postSave() {
		$this->removeCache( $this->getId() );

		$cbf = new ClientBalanceFactory();
		$cbf->initializeBalance( $this->getId() );

		return TRUE;
	}

	function addLog( $log_action ) {
		return TTLog::addEntry( $this->getId(), $log_action, 'Client Information' , NULL, $this->getTable() );
	}

}
?>
