<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 418 $
 * $Id: BranchFactory.class.php 418 2006-12-07 23:28:14Z ipso $
 * $Date: 2006-12-07 15:28:14 -0800 (Thu, 07 Dec 2006) $
 */
class BranchFactory extends Factory {
	protected $table = 'branch';
	protected $pk_sequence_name = 'branch_id_seq'; //PK Sequence name

	protected $status_options = array(
										10 => 'ENABLED',
										20 => 'DISABLED'
									);

	protected $company_validator_msg = 'Company is invalid';
	protected $company_validator_label = 'company';

	protected $status_validator_options_msg = 'Incorrect Status';
	protected $status_validator_label = 'status';

	protected $name_validator_length_msg = 'Name is too short or too long';
	protected $name_validator_unique_msg = 'Branch already exists';
	protected $name_validator_label = 'name';

	protected $address_validator_regex = '/^[a-zA-Z0-9-_\.#\ ]{1,250}$/i';

	protected $address1_validator_regex_msg = 'Address1 contains invalid characters';
	protected $address1_validator_length_msg = 'Address1 is too short or too long';
	protected $address1_validator_label = 'address1';

	protected $address2_validator_regex_msg = 'Address2 contains invalid characters';
	protected $address2_validator_length_msg = 'Address2 is too short or too long';
	protected $address2_validator_label = 'address2';

	protected $city_validator_regex = '/^[a-zA-Z0-9-\.\ ]{1,250}$/i';
	protected $city_validator_regex_msg = 'City contains invalid characters';
	protected $city_validator_length_msg = 'City name is too short or too long';
	protected $city_validator_label = 'city';

	protected $province_validator_options_msg = 'Invalid Province';
	protected $province_validator_label = 'province';

	protected $country_validator_options_msg = 'Invalid Country';
	protected $country_validator_label = 'country';

	protected $postalcode_validator_regex_msg = 'Postal Code contains invalid characters, or invalid format';
	protected $postalcode_validator_length_msg = 'Postal Code is too short or too long';
	protected $postalcode_validator_label = 'postal_code';

	protected $workphone_validator_isphonenumber_msg = 'Work phone number is invalid';
	protected $workphone_validator_label = 'work_phone';

	protected $faxphone_validator_isphonenumber_msg = 'Fax phone number is invalid';
	protected $faxphone_validator_label = 'fax_phone';

	function getCompany() {
		return $this->data['company_id'];
	}
	function setCompany($id) {
		$id = trim($id);

		$clf = new CompanyListFactory();

		if ( $id == 0
				OR $this->Validator->isResultSetWithRows(	$this->company_validator_label,
															$clf->getByID($id),
															$this->company_validator_msg
															) ) {
			$this->data['company_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getStatus() {
		//Have to return the KEY because it should always be a drop down box.
		//return Option::getByKey($this->data['status_id'], $this->getOptions('status') );
		return $this->data['status_id'];
	}
	function setStatus($status) {
		$status = trim($status);

		$key = Option::getByValue($status, $this->getOptions('status') );
		if ($key !== FALSE) {
			$status = $key;
		}

		if ( $this->Validator->inArrayKey(	$this->status_validator_label,
											$status,
											$this->status_validator_options_msg,
											$this->getOptions('status')) ) {

			$this->data['status_id'] = $status;

			return FALSE;
		}

		return FALSE;
	}

	function isUniqueName($name) {
		$ph = array(
					'company_id' => $this->getCompany(),
					'name' => $name,
					);

		$query = 'select id from '. $this->getTable() .'
					where company_id = ?
						AND name = ?';
		$name_id = $this->db->GetOne($query, $ph);
		Debug::Arr($name_id,'Unique Name: '. $name, __FILE__, __LINE__, __METHOD__,10);

		if ( $name_id === FALSE ) {
			return TRUE;
		} else {
			if ($name_id == $this->getId() ) {
				return TRUE;
			}
		}

		return FALSE;
	}

	function getName() {
		return $this->data['name'];
	}
	function setName($name) {
		$name = trim($name);

		if 	(	$this->Validator->isLength(		$this->name_validator_label,
												$name,
												$this->name_validator_length_msg,
												2,
												100)
					AND
						$this->Validator->isTrue(		$this->name_validator_label,
														$this->isUniqueName($name),
														$this->name_validator_unique_msg)

												) {

			$this->data['name'] = $name;

			return TRUE;
		}

		return FALSE;
	}

	function getAddress1() {
		return $this->data['address1'];
	}
	function setAddress1($address1) {
		$address1 = trim($address1);

		if 	(	$this->Validator->isRegEx(		$this->address1_validator_label,
												$address1,
												$this->address1_validator_regex_msg,
												$this->address_validator_regex)
				AND
					$this->Validator->isLength(		$this->address1_validator_label,
													$address1,
													$this->address1_validator_length_msg,
													2,
													250) ) {

			$this->data['address1'] = $address1;

			return TRUE;
		}

		return FALSE;
	}

	function getAddress2() {
		return $this->data['address2'];
	}
	function setAddress2($address2) {
		$address2 = trim($address2);

		if 	(	$address2 != NULL
				AND (
					$this->Validator->isRegEx(		$this->address2_validator_label,
													$address2,
													$this->address2_validator_regex_msg,
													$this->address_validator_regex)
				AND
					$this->Validator->isLength(		$this->address2_validator_label,
													$address2,
													$this->address2_validator_length_msg,
													2,
													250) ) ) {

			$this->data['address2'] = $address2;

			return TRUE;
		}

		return FALSE;

	}

	function getCity() {
		return $this->data['city'];
	}
	function setCity($city) {
		$city = trim($city);

		if 	(	$this->Validator->isRegEx(		$this->city_validator_label,
												$city,
												$this->city_validator_regex_msg,
												$this->city_validator_regex)
				AND
					$this->Validator->isLength(		$this->city_validator_label,
													$city,
													$this->city_validator_length_msg,
													2,
													250) ) {

			$this->data['city'] = $city;

			return TRUE;
		}

		return FALSE;
	}

	function getProvince() {
		if ( isset($this->data['province']) ) {
			return $this->data['province'];
		}

		return FALSE;
	}
	function setProvince($province) {
		$province = trim($province);

		Debug::Text('Country: '. $this->getCountry() .' Province: '. $province, __FILE__, __LINE__, __METHOD__,10);

		$cf = new CompanyFactory();

		$options_arr = $cf->getOptions('province');
		if ( isset($options_arr[$this->getCountry()]) ) {
			$options = $options_arr[$this->getCountry()];
		} else {
			$options = array();
		}

		if ( $this->Validator->inArrayKey(		$this->province_validator_label,
												$province,
												$this->province_validator_options_msg,
												$options ) ) {

			$this->data['province'] = $province;

			return TRUE;
		}

		return FALSE;
	}

	function getCountry() {
		if ( isset($this->data['country']) ) {
			return $this->data['country'];
		}

		return FALSE;
	}
	function setCountry($country) {
		$country = trim($country);

		$cf = new CompanyFactory();

		if ( $this->Validator->inArrayKey(		$this->country_validator_label,
												$country,
												$this->country_validator_options_msg,
												$cf->getOptions('country') ) ) {

			$this->data['country'] = $country;

			return TRUE;
		}

		return FALSE;
	}

	function getPostalCode() {
		return $this->data['postal_code'];
	}
	function setPostalCode($postal_code) {
		$postal_code = strtoupper( $this->Validator->stripSpaces($postal_code) );

		if 	(
				$postal_code == ''
				OR
				(
				$this->Validator->isPostalCode(		$this->postalcode_validator_label,
													$postal_code,
													$this->postalcode_validator_regex_msg,
													$this->getCountry() )
				AND
					$this->Validator->isLength(		$this->postalcode_validator_label,
													$postal_code,
													$this->postalcode_validator_length_msg,
													1,
													10)
				)
				) {

			$this->data['postal_code'] = $postal_code;

			return TRUE;
		}

		return FALSE;
	}

	function getWorkPhone() {
		return $this->data['work_phone'];
	}
	function setWorkPhone($work_phone) {
		$work_phone = trim($work_phone);

		if 	(	$this->Validator->isPhoneNumber(		$this->workphone_validator_label,
														$work_phone,
														$this->workphone_validator_isphonenumber_msg) ) {

			$this->data['work_phone'] = $work_phone;

			return TRUE;
		}

		return FALSE;
	}

	function getFaxPhone() {
		return $this->data['fax_phone'];
	}
	function setFaxPhone($fax_phone) {
		$fax_phone = trim($fax_phone);

		if 	(	$this->Validator->isPhoneNumber(		$this->faxphone_validator_label,
														$fax_phone,
														$this->faxphone_validator_isphonenumber_msg) ) {

			$this->data['fax_phone'] = $fax_phone;

			return TRUE;
		}

		return FALSE;
	}

	function postSave() {
		$this->removeCache( $this->getId() );

		return TRUE;
	}

	function addLog( $log_action ) {
		return TTLog::addEntry( $this->getId(), $log_action, 'Branch', NULL, $this->getTable() );
	}

}
?>
