<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 417 $
 * $Id: Wage.class.php 417 2006-12-06 22:58:53Z ipso $
 * $Date: 2006-12-06 14:58:53 -0800 (Wed, 06 Dec 2006) $
 */
class Wage {
	var $user_id = NULL;
	var $pay_period_id = NULL;
	var $advance = FALSE;

	var $user_date_total_arr = NULL;

	var $user_obj = NULL;
	var $user_tax_obj = NULL;
	var $user_wage_obj = NULL;
	var $user_pay_period_total_obj = NULL;
	var $pay_stub_entry_account_link_obj = NULL;

	var $pay_period_obj = NULL;
	var $pay_period_schedule_obj = NULL;

	var $labor_standard_obj = NULL;
	var $holiday_obj = NULL;

	function __construct($user_id, $pay_period_id) {
		/*
		Debug::text('Wage Class Constructor! User ID: '. $user_id, __FILE__, __LINE__, __METHOD__,10);
		Debug::text('-----------------------------------------------------------------------', __FILE__, __LINE__, __METHOD__,10);
		Debug::Arr(debug_backtrace(), 'Debug BackTrace', __FILE__, __LINE__, __METHOD__,10);
		debug_print_backtrace();
		*/

		$this->user_id = $user_id;
		$this->pay_period_id = $pay_period_id;

		return TRUE;
	}

	function getUser() {
		return $this->user_id;
	}

	function getPayPeriod() {
		return $this->pay_period_id;
	}

	function getAdvance() {
		if ( isset($this->advance) ) {
			return $this->advance;
		}

		return FALSE;
	}
	function setAdvance($bool) {
		$this->advance = $bool;

		return TRUE;
	}

	function getUserObject() {
		if ( is_object($this->user_obj) ) {
			return $this->user_obj;
		} else {
			$ulf = new UserListFactory();
			$this->user_obj = $ulf->getById( $this->getUser() )->getCurrent();

			return $this->user_obj;
		}
	}

	function getPayStubEntryAccountLinkObject() {
		if ( is_object($this->pay_stub_entry_account_link_obj) ) {
			return $this->pay_stub_entry_account_link_obj;
		} else {
			$pseallf = new PayStubEntryAccountLinkListFactory();
			$pseallf->getByCompanyId( $this->getUserObject()->getCompany() );
			if ( $pseallf->getRecordCount() > 0 ) {
				$this->pay_stub_entry_account_link_obj = $pseallf->getCurrent();
				return $this->pay_stub_entry_account_link_obj;
			}

			return FALSE;
		}
	}

	function getUserTaxObject() {
		if ( is_object($this->user_tax_obj) ) {
			return $this->user_tax_obj;
		} else {

			$utlf = new UserTaxListFactory();
			$utlf->getByUserId( $this->getUser() );
			if ( $utlf->getRecordCount() > 0 ) {
				$this->user_tax_obj = $utlf->getCurrent();

				return $this->user_tax_obj;
			}

			return FALSE;
		}
	}
	
	function getUserWageObject( $user_wage_id ) {
		if ( isset($this->user_wage_obj[$user_wage_id])
			AND is_object($this->user_wage_obj[$user_wage_id]) ) {
			return $this->user_wage_obj[$user_wage_id];
		} else {
			$uwlf = new UserWageListFactory();

			//$this->user_wage_obj = $uwlf->getLastWageByUserId( $this->getUser() )->getCurrent();

			//This handles future wage changes properly.
			//$this->user_wage_obj = $uwlf->getWageByUserIdAndPayPeriodEndDate( $this->getUser(), $this->getPayPeriodObject()->getEndDate() )->getCurrent();
			$uwlf->getByID( $user_wage_id );
			if ( $uwlf->getRecordCount() > 0 ) {
				$this->user_wage_obj[$user_wage_id] = $uwlf->getCurrent();

				return $this->user_wage_obj[$user_wage_id];
			}

			return FALSE;
		}
	}

	function getPayPeriodObject() {
		if ( is_object($this->pay_period_obj) ) {
			return $this->pay_period_obj;
		} else {
			$pplf = new PayPeriodListFactory();
			//$this->pay_period_obj = $pplf->getById( $this->getPayPeriod() )->getCurrent();
			$pplf->getById( $this->getPayPeriod() );
			if ( $pplf->getRecordCount() > 0 ) {
				$this->pay_period_obj = $pplf->getCurrent();

				return $this->pay_period_obj;
			}

			return FALSE;
		}
	}

	function getPayPeriodScheduleObject() {
		if ( is_object($this->pay_period_schedule_obj) ) {
			return $this->pay_period_schedule_obj;
		} else {
			$ppslf = new PayPeriodScheduleListFactory();
			$this->pay_period_schedule_obj = $ppslf->getById( $this->getPayPeriodObject()->getPayPeriodSchedule() )->getCurrent();

			return $this->pay_period_schedule_obj;
		}
	}

	function getLaborStandardObject() {
		if ( is_object($this->labor_standard_obj) ) {
			return $this->labor_standard_obj;
		} else {
			$ls = new LaborStandard();
			$ls->getByCountryAndProvince( $this->getUserObject()->getCountry() , $this->getUserObject()->getProvince() );
			$this->labor_standard_obj = $ls;

			return $this->labor_standard_obj;
		}
	}

	static function getRemittanceDueDate($transaction_epoch, $avg_monthly_remittance) {
		Debug::text('Transaction Date: '. TTDate::getDate('DATE+TIME', $transaction_epoch) , __FILE__, __LINE__, __METHOD__,10);

		if ( $avg_monthly_remittance < 15000 ) {
			Debug::text('Regular Monthly' , __FILE__, __LINE__, __METHOD__,10);
			//15th of the month FOLLOWING transaction_epoch.
			$due_date = mktime(0,0,0,date('n',$transaction_epoch)+1,15,date('y',$transaction_epoch) );
		} elseif ( $avg_monthly_remittance >= 15000 AND $avg_monthly_remittance < 49999.99 ) {
			Debug::text('Accelerated Threshold 1' , __FILE__, __LINE__, __METHOD__,10);
			/*
			Amounts you deduct or withhold from remuneration paid in the first 15 days of the month
			are due by the 25th of the same month. Amounts you withhold from the 16th to the end of
			the month are due by the 10th day of the following month.
			*/
			if ( date('j', $transaction_epoch) <= 15 ) {
				$due_date = mktime(0,0,0,date('n',$transaction_epoch),25,date('y',$transaction_epoch) );
			} else {
				$due_date = mktime(0,0,0,date('n',$transaction_epoch)+1,10,date('y',$transaction_epoch) );
			}
		} elseif ( $avg_monthly_remittance > 50000) {
			Debug::text('Accelerated Threshold 2' , __FILE__, __LINE__, __METHOD__,10);
			/*
			Amounts you deduct or withhold from remuneration you pay any time during the month are due by the third working day (not counting Saturdays, Sundays, or holidays) after the end of the following periods:

			* from the 1st through the 7th day of the month;
			* from the 8th through the 14th day of the month;
			* from the 15th through the 21st day of the month; and
			* from the 22nd through the last day of the month.
			*/
			return FALSE;
		}

		return $due_date;
	}

	//Round advances to the nearest $25, in hopes of the value not changing much.
	function roundAdvance($value) {
		Debug::text('Before Round: '. $value , __FILE__, __LINE__, __METHOD__,10);

		$tmp_value = round($value);
		//$retval = $tmp_value - ( $tmp_value % 50); //Round down by $50.
		$retval = bcsub($tmp_value, bcmod( $tmp_value, 50) ); //Round down by $50.

		if ($retval < 0 ) {
			$retval = $value;
		}

		Debug::text('After Round: '. $retval , __FILE__, __LINE__, __METHOD__,10);

		return $retval;
	}


	function getHourlyRate( $user_wage_id ) {
		if ( is_object($this->getUserWageObject( $user_wage_id ) ) ) {
			return $this->getUserWageObject( $user_wage_id )->getHourlyRate( FALSE, FALSE, $this->getPayPeriodObject()->getStartDate() );
		}

		return 0;
	}

	function getAnnualHourlyRate( $user_wage_id, $exclude_stats = FALSE, $exclude_vacation = FALSE ) {
		if ( is_object($this->getUserWageObject( $user_wage_id ) ) ) {
			return $this->getUserWageObject( $user_wage_id )->getAnnualHourlyRate( $exclude_stats, $exclude_vacation, $this->getPayPeriodObject()->getStartDate() );
		}

		return 0;
	}

	function getWage($seconds, $rate ) {
		if ( $seconds == '' OR empty($seconds) ) {
			return 0;
		}

		if ( $rate == '' OR empty($rate) ) {
			return 0;
		}

		//return TTDate::getHours( $seconds ) * $rate;
		return bcmul( TTDate::getHours( $seconds ), $rate );
	}

	function getMaximumPayPeriodWage( $user_wage_id ) {
		if ( $this->getAdvance() == TRUE ) {
			Debug::text('Absolute Maximum Pay Period Advance is TRUE: Annual Pay Periods: '. $this->getPayPeriodScheduleObject()->getAnnualPayPeriods(), __FILE__, __LINE__, __METHOD__,10);
			//$maximum_pay_period_wage = $this->getUserWageObject( $user_wage_id )->getWage() / ( $this->getPayPeriodScheduleObject()->getAnnualPayPeriods() * 2);
			$maximum_pay_period_wage = bcdiv( $this->getUserWageObject( $user_wage_id )->getWage(), bcmul( $this->getPayPeriodScheduleObject()->getAnnualPayPeriods(), 2 ) );
		} else {
			Debug::text('Absolute Maximum Pay Period NO Advance: Annual Pay Periods: '. $this->getPayPeriodScheduleObject()->getAnnualPayPeriods(), __FILE__, __LINE__, __METHOD__,10);
			//$maximum_pay_period_wage = $this->getUserWageObject( $user_wage_id )->getWage() / $this->getPayPeriodScheduleObject()->getAnnualPayPeriods();
			$maximum_pay_period_wage = bcdiv( $this->getUserWageObject( $user_wage_id )->getWage(), $this->getPayPeriodScheduleObject()->getAnnualPayPeriods() );
		}
		Debug::text('Absolute Maximum Pay Period Wage: '. $maximum_pay_period_wage, __FILE__, __LINE__, __METHOD__,10);

		return $maximum_pay_period_wage;
	}


	function getStatutoryWage() {
		//BC includes commisions/bonuses
		//AB does not.
		//AB's average statutory days is incorrect, should be 9weeks?

		//To include commissions. Calculate average daily wage (not including over time)
		//Then add up all pay stub amendments in a 30day time period and include that.

		return FALSE;
/*
		//FIXME: split shifts will skew this amount.

		$psalf = new PayStubAmendmentListFactory();
		$slf = new ShiftListFactory();

		//Get all holidays in this pay period.
		$pay_period_start_date = TTDate::getBeginDayEpoch( $this->getPayPeriodObject()->getStartDate() );
		$pay_period_end_date = $this->getPayPeriodObject()->getEndDate();

		$holiday_epochs = $this->getHolidayObject()->getHolidayEpochs( $pay_period_start_date );
		foreach($holiday_epochs as $name => $epoch) {
			if ( $epoch <= $pay_period_end_date) {
				if ( $this->getLaborStandardObject()->getStatutoryEligibility( $this->getUserObject()->getId(), $epoch ) == TRUE ) {
					$pay_period_holidays[] = $epoch;
				} else {
					Debug::text('Not eligable for this stat day.. ', __FILE__, __LINE__, __METHOD__,10);
				}

			}
		}

		if ( isset($pay_period_holidays) ) {
			$total_pay_period_holidays = count($pay_period_holidays);
		} else {
			$total_pay_period_holidays = 0;
		}

		Debug::text('Found Holidays in pay period: '. $total_pay_period_holidays , __FILE__, __LINE__, __METHOD__,10);
		if ( $total_pay_period_holidays > 0 ){
			Debug::text('Calculating pay for each stat day. ', __FILE__, __LINE__, __METHOD__,10);

			$statutory_wage = 0;
			foreach($pay_period_holidays as $pay_period_holiday_epoch) {

				$average_daily_time = TTDate::getHours( $this->getLaborStandardObject()->getAverageDailyTime( $this->getUserObject()->getId(), ($pay_period_holiday_epoch - ($this->getLaborStandardObject()->getStatutoryAverageDays() * 86400)), $pay_period_holiday_epoch ) );
				Debug::text('Average Daily Time for Stat: '. $average_daily_time, __FILE__, __LINE__, __METHOD__,10);

				if ( $this->getLaborStandardObject()->getStatutoryPayIncludesBonus() == 1 ) {
					Debug::text('Statutory Pay includes commissions/bonuses ', __FILE__, __LINE__, __METHOD__,10);

					//FIXME: exclude certain one time bonuses from this amount.
					$pay_stub_amendment_sum = $psalf->getAmountSumByUserIdAndTypeIdAndTaxExemptAndAuthorizedAndStartDateAndEndDate( $this->getUserObject()->getId(), 10, FALSE, TRUE, ($pay_period_holiday_epoch - ($this->getLaborStandardObject()->getStatutoryAverageDays() * 86400)), $pay_period_holiday_epoch );
					Debug::text('Full Pay Stub Amendments Sum: '. $pay_stub_amendment_sum, __FILE__, __LINE__, __METHOD__,10);

					$last_days_worked_count = $slf->getDaysWorkedByStartDateAndEndDate( $this->getUserObject()->getId(), ( $pay_period_holiday_epoch - ( $this->getLaborStandardObject()->getStatutoryEmployedPeriod() * 86400 ) ), $pay_period_holiday_epoch );
					Debug::text('Last Days worked for stat averaging: '. $last_days_worked_count, __FILE__, __LINE__, __METHOD__,10);

					$pay_stub_amendment_amount = $pay_stub_amendment_sum / $last_days_worked_count;
					Debug::text('Averaged Pay Stub Amendment Amount: '. $pay_stub_amendment_amount, __FILE__, __LINE__, __METHOD__,10);

					$hourly_rate = $this->getHourlyRate( TRUE );
				} else {
					Debug::text('Statutory Pay DOES NOT includes commissions/bonuses ', __FILE__, __LINE__, __METHOD__,10);
					$pay_stub_amendment_amount = 0;

					$hourly_rate = $this->getHourlyRate();
				}

				$statutory_wage += ($average_daily_time * $hourly_rate ) + $pay_stub_amendment_amount;
				Debug::text('Statutory Wage: '. $statutory_wage .' Hourly Rate: '. $hourly_rate, __FILE__, __LINE__, __METHOD__,10);
			}

			Debug::text('Total Statutory Wage: '. $statutory_wage, __FILE__, __LINE__, __METHOD__,10);
			return $statutory_wage;
		}

		return FALSE;
*/
	}

	function getPayStubAmendmentEarnings() {
		//Get pay stub amendments here.
		$psalf = new PayStubAmendmentListFactory();

		if ( $this->getAdvance() == TRUE ) {
			//For advances, any PS amendment effective BEFORE the advance end date is considered in full.
			//Any AFTER the advance end date, is considered half.

			//$pay_period_end_date = $this->getPayPeriodObject()->getAdvanceEndDate();
			$advance_pos_sum = $psalf->getAmountSumByUserIdAndTypeIdAndAuthorizedAndStartDateAndEndDate( $this->getUser(), 10, TRUE, $this->getPayPeriodObject()->getStartDate(), $this->getPayPeriodObject()->getAdvanceEndDate() );
			Debug::text('Pay Stub Amendment Advance Earnings: '. $advance_pos_sum , __FILE__, __LINE__, __METHOD__,10);

			$full_pos_sum = $psalf->getAmountSumByUserIdAndTypeIdAndAuthorizedAndStartDateAndEndDate( $this->getUser(), 10, TRUE, $this->getPayPeriodObject()->getAdvanceEndDate(), $this->getPayPeriodObject()->getEndDate() );
			Debug::text('Pay Stub Amendment Full Earnings: '. $full_pos_sum , __FILE__, __LINE__, __METHOD__,10);
			//Take the full amount of PS amendments BEFORE the advance end date, and half of any AFTER the advance end date.
			//$pos_sum = $advance_pos_sum + ($full_pos_sum / 2);
			$pos_sum = bcadd( $advance_pos_sum, bcdiv( $full_pos_sum, 2 ) );
		} else {
			//$pay_period_end_date =
			$pos_sum = $psalf->getAmountSumByUserIdAndTypeIdAndAuthorizedAndStartDateAndEndDate( $this->getUser(), 10, TRUE, $this->getPayPeriodObject()->getStartDate(), $this->getPayPeriodObject()->getEndDate() );
		}
		//$neg_sum = $psalf->getAmountSumByUserIdAndTypeIdAndTaxExemptAndAuthorizedAndStartDateAndEndDate( $this->getUser(), 20, FALSE, TRUE, $this->getPayPeriodObject()->getStartDate(), $this->getPayPeriodObject()->getEndDate() )*-1;

		Debug::text('Pay Stub Amendment Total Earnings: '. $pos_sum , __FILE__, __LINE__, __METHOD__,10);

		return $pos_sum;
	}

	function getPayStubAmendmentDeductions() {
		//Get pay stub amendments here.
		$psalf = new PayStubAmendmentListFactory();

		if ( $this->getAdvance() == TRUE ) {
			//For advances, any PS amendment effective BEFORE the advance end date is considered in full.
			//Any AFTER the advance end date, is considered half.

			//$pay_period_end_date = $this->getPayPeriodObject()->getAdvanceEndDate();
			$advance_neg_sum = $psalf->getAmountSumByUserIdAndTypeIdAndAuthorizedAndStartDateAndEndDate( $this->getUser(), 20, TRUE, $this->getPayPeriodObject()->getStartDate(), $this->getPayPeriodObject()->getAdvanceEndDate() );
			Debug::text('Pay Stub Amendment Advance Deductions: '. $advance_neg_sum , __FILE__, __LINE__, __METHOD__,10);

			$full_neg_sum = $psalf->getAmountSumByUserIdAndTypeIdAndAuthorizedAndStartDateAndEndDate( $this->getUser(), 20, TRUE, $this->getPayPeriodObject()->getAdvanceEndDate(), $this->getPayPeriodObject()->getEndDate() );
			Debug::text('Pay Stub Amendment Full Deductions: '. $full_neg_sum , __FILE__, __LINE__, __METHOD__,10);
			//Take the full amount of PS amendments BEFORE the advance end date, and half of any AFTER the advance end date.
			//$neg_sum = $advance_neg_sum + ($full_neg_sum / 2);
			$neg_sum = bcadd( $advance_neg_sum, bcdiv( $full_neg_sum, 2 ) );
		} else {
			//$pay_period_end_date =
			$neg_sum = $psalf->getAmountSumByUserIdAndTypeIdAndAuthorizedAndStartDateAndEndDate( $this->getUser(), 20, TRUE, $this->getPayPeriodObject()->getStartDate(), $this->getPayPeriodObject()->getEndDate() );
		}
		//$neg_sum = $psalf->getAmountSumByUserIdAndTypeIdAndTaxExemptAndAuthorizedAndStartDateAndEndDate( $this->getUser(), 20, FALSE, TRUE, $this->getPayPeriodObject()->getStartDate(), $this->getPayPeriodObject()->getEndDate() )*-1;


		Debug::text('Pay Stub Amendment Total Deductions: '. $neg_sum , __FILE__, __LINE__, __METHOD__,10);

		return bcmul( $neg_sum, -1 );
	}

	function getPayStubAmendmentWage() {

		$sum = bcadd( $this->getPayStubAmendmentEarnings(), $this->getPayStubAmendmentDeductions() );

		/*
		//We have a choice. Take in to account ALL amendments for the pay period, divide them in
        //half for the advance. Or just take in to account amendments BEFORE the advance end date,
        //and include 100% of those.

        //Take in to account pay stub amendments...
		if ( $this->getAdvance() == TRUE ) {
			//Cut amendments in half if this it the advance pay stub.
			Debug::text('Pay Stub Amendment Wage, cut in half due to advance.', __FILE__, __LINE__, __METHOD__,10);
			$sum = $sum / 2;
		}
		*/
		Debug::text('Pay Stub Amendment Wage: '. $sum , __FILE__, __LINE__, __METHOD__,10);

		return $sum;
	}
/*
	function getPayStubAmendmentVacationAmount() {
		$psalf = new PayStubAmendmentListFactory();
		//$advance_neg_sum = $psalf->getAmountSumByUserIdAndTypeIdAndTaxExemptAndAuthorizedAndStartDateAndEndDate( $this->getUser(), 20, $tax_exempt, TRUE, $this->getPayPeriodObject()->getStartDate(), $this->getPayPeriodObject()->getAdvanceEndDate() );
		$vacation_amount = $psalf->getAmountSumByUserIdAndNameIdAndTaxExemptAndAuthorizedAndStartDateAndEndDate( $this->getUser(), $this->getPayStubEntryAccountLinkObject()->getVacationAccrual(), FALSE, TRUE, $this->getPayPeriodObject()->getStartDate(), $this->getPayPeriodObject()->getEndDate() );

		//Always include vacation accrual release in this.
		$vacation_accrual_release_amount = $psalf->getAmountSumByUserIdAndNameIdAndTaxExemptAndAuthorizedAndStartDateAndEndDate( $this->getUser(), $this->getPayStubEntryAccountLinkObject()->getVacationAccrualRelease(), FALSE, TRUE, $this->getPayPeriodObject()->getStartDate(), $this->getPayPeriodObject()->getEndDate() );

		Debug::text('Pay Stub Amendment Vacation Amount: '. $vacation_amount .' Accrual Amount: '. $vacation_accrual_release_amount .' User: '. $this->getUser() , __FILE__, __LINE__, __METHOD__,10);

		//return $vacation_amount + $vacation_accrual_release_amount;
		return bcadd($vacation_amount, $vacation_accrual_release_amount);
	}

	function getVacationAccrual() {
		if ( is_object( $this->getUserTaxObject() ) ) {
			//$rate = $this->getUserTaxObject()->getVacationRate() / 100;
			$rate = bcdiv( $this->getUserTaxObject()->getVacationRate(), 100 );
		} else {
			Debug::text('User Tax Object Not set, returning 0 ', __FILE__, __LINE__, __METHOD__,10);
		}

		if ( !isset($rate) ) {
			Debug::text('Vacation Rate Not set, returning 0 ', __FILE__, __LINE__, __METHOD__,10);
			return 0;
		}
		Debug::text('Vacation Rate: '. $rate, __FILE__, __LINE__, __METHOD__,10);

		$user_date_total_arr = $this->getUserDateTotalArray();

		//Don't calculate vacation on top of vacation pay.
		//Get any PS amendments that might modify vacation pay.

		//User amounts, not time here... Because of wage change mid pay periods.
		//$additional_vacation_pay = $this->getPayStubAmendmentVacationAmount();
		//$additional_vacation_pay = $this->getPayStubAmendmentVacationAmount() + ( TTDate::getHours($user_date_total_arr['other']['paid_vacation_time']) * $this->getHourlyRate() );
		//$additional_vacation_pay = $this->getPayStubAmendmentVacationAmount() + $user_date_total_arr['other']['paid_vacation_amount'];
		$additional_vacation_pay = bcadd( $this->getPayStubAmendmentVacationAmount(), $user_date_total_arr['other']['paid_vacation_amount'] );
		Debug::text('Reducing Gross Wage by Vacation Amount Released: '. $additional_vacation_pay, __FILE__, __LINE__, __METHOD__,10);

		//$vacation_pay = ( $this->getGrossWage( FALSE, FALSE ) - $additional_vacation_pay ) * $rate;
		$vacation_pay = bcmul( bcsub( $this->getGrossWage( FALSE, FALSE ), $additional_vacation_pay ), $rate );

		Debug::text('Vacation Pay: '. $vacation_pay .' At Rate: '. $rate, __FILE__, __LINE__, __METHOD__,10);

		if ( $vacation_pay < 0 ) {
			$vacation_pay = 0;
		}

		return $vacation_pay;
	}
*/

	function getRawGrossWage() {
		$wage = 0;

		$udt_arr = $this->getUserDateTotalArray();
		if ( isset($udt_arr['entries']) AND count($udt_arr['entries']) > 0 ) {
			foreach( $udt_arr['entries'] as $udt ) {
				if ( isset($udt['amount']) ) {
					$wage += $udt['amount'];
				}
			}
		}

		Debug::text('Raw Gross Wage: '. $wage , __FILE__, __LINE__, __METHOD__,10);
		return $wage;
	}

	//function getGrossWage( $include_vacation = TRUE, $include_tax_exempt_deductions = TRUE ) {
	function getGrossWage() {

		$wage = $this->getRawGrossWage();

		Debug::text('Gross Wage (NOT incl amendments) $'. $wage , __FILE__, __LINE__, __METHOD__,10);

		return $wage;

		//Get auto - vacation release amount
		//
		//WATCHOUT. getVacationAccrual() calls getGrossWage and causes an infinite loop and segfaults PHP
		//
		/*
		if ( $include_vacation == TRUE
				AND is_object( $this->getUserTaxObject() )
				AND $this->getUserTaxObject()->getReleaseVacation() == TRUE ) {
			$vacation_accrual = $this->getVacationAccrual();
			Debug::text('Vacation Accrual Release Amount: '. $vacation_accrual , __FILE__, __LINE__, __METHOD__,10);
			//$wage += $vacation_accrual;
			$wage = bcadd( $wage, $vacation_accrual );
		}


		$ps_amendment_amount = $this->getPayStubAmendmentEarnings();
		Debug::text('aGross Wage (PS Amendment Amount) $'. $ps_amendment_amount , __FILE__, __LINE__, __METHOD__,10);

		if ( $include_tax_exempt_deductions == TRUE ) {
			//$ps_amendment_amount += $this->getPayStubAmendmentDeductions(TRUE);
			$ps_amendment_amount = bcadd( $ps_amendment_amount, $this->getPayStubAmendmentDeductions(TRUE) );
		}

		Debug::text('bGross Wage (PS Amendment Amount) $'. $ps_amendment_amount , __FILE__, __LINE__, __METHOD__,10);


		//$wage += $ps_amendment_amount;
		$wage = bcadd( $wage, $ps_amendment_amount );

		Debug::text('Gross Wage (incl. Bonus) $'. $wage , __FILE__, __LINE__, __METHOD__,10);

		return $wage;
		*/
	}

	function getUserDateTotalArray() {
		if ( isset($this->user_date_total_arr) ) {
			return $this->user_date_total_arr;
		}

		//If the user date total array isn't set, set it now, and return its value.
		return $this->setUserDateTotalArray();
		//return FALSE;
	}

	function setUserDateTotalArray() {

		//Loop through unique UserDateTotal rows... Adding entries to pay stubs.
		$udtlf = new UserDateTotalListFactory();
		if ( $this->getAdvance() == TRUE ) {
			$udtlf->getByUserIdAndPayPeriodIdAndEndDate( $this->getUser(), $this->getPayPeriod(), $this->getPayPeriodObject()->getAdvanceEndDate() );
		} else {
			$udtlf->getByUserIdAndPayPeriodIdAndEndDate( $this->getUser(), $this->getPayPeriod(), $this->getPayPeriodObject()->getEndDate() );
		}

		$dock_absence_time = 0;
		$paid_absence_time = 0;
		$paid_vacation_time = 0;
		$dock_absence_amount = 0;
		$paid_absence_amount = 0;
		$paid_vacation_amount = 0;
		$prev_wage_effective_date = 0;
		if ( $udtlf->getRecordCount() > 0 ) {
			foreach( $udtlf as $udt_obj ) {
				Debug::text('User Total Row... Type: '. $udt_obj->getType() .' OverTime Policy ID: '. $udt_obj->getOverTimePolicyID() .' User Wage ID: '. $udt_obj->getColumn('user_wage_id') , __FILE__, __LINE__, __METHOD__,10);

				if ( $udt_obj->getStatus() == 10 AND $udt_obj->getType() == 20 ) { //Regular Time
					Debug::text('User Total Row... Regular Time: '. $udt_obj->getTotalTime()  , __FILE__, __LINE__, __METHOD__,10);

					//Check if they are a salary user...
					//Use WORKED time to calculate regular time. Not just regular time.
					if ( is_object($this->getUserWageObject( $udt_obj->getColumn('user_wage_id') ))
							AND $this->getUserWageObject( $udt_obj->getColumn('user_wage_id') )->getType() == 20 ) {
						//Salary
						Debug::text('Strict Salary Wage: Reduce Regular Pay By: Dock Time: '. $dock_absence_time .' and Paid Absence: '. $paid_absence_time, __FILE__, __LINE__, __METHOD__,10);

						//$dock_absence_wage = $this->getWage( $dock_absence_time, $this->getHourlyRate( $udt_obj->getColumn('user_wage_id') ) );
						//$paid_absence_wage = $this->getWage( $paid_absence_time, $this->getHourlyRate( $udt_obj->getColumn('user_wage_id') ) );
						if ( isset($dock_absence_amount_arr[$udt_obj->getColumn('user_wage_id')]) ) {
							$dock_absence_wage = $dock_absence_amount_arr[$udt_obj->getColumn('user_wage_id')];
						} else {
							$dock_absence_wage = 0;
						}
						if ( isset($paid_absence_amount_arr[$udt_obj->getColumn('user_wage_id')]) ) {
							$paid_absence_wage = $paid_absence_amount_arr[$udt_obj->getColumn('user_wage_id')];
						} else {
							$paid_absence_wage = 0;
						}
						Debug::text('Wage ID: '. $udt_obj->getColumn('user_wage_id') .' Dock Absence Wage: '. $dock_absence_wage .' Paid Absence Wage: '. $paid_absence_wage, __FILE__, __LINE__, __METHOD__,10);

						$maximum_wage_salary = UserWageFactory::proRateSalary( $this->getMaximumPayPeriodWage( $udt_obj->getColumn('user_wage_id') ) , $this->getUserWageObject( $udt_obj->getColumn('user_wage_id') )->getEffectiveDate(), $prev_wage_effective_date, $this->getPayPeriodObject()->getStartDate(), $this->getPayPeriodObject()->getEndDate() );

						$prev_wage_effective_date = $this->getUserWageObject( $udt_obj->getColumn('user_wage_id') )->getEffectiveDate();

						//$pay_stub->addEntry('regular', , TTDate::getHours( $udt_obj->getTotalTime() ), NULL );
						//$amount = $maximum_wage_salary - ( $dock_absence_wage + $paid_absence_wage );
						$amount = bcsub( $maximum_wage_salary, bcadd( $dock_absence_wage, $paid_absence_wage ) );
						$total_time = $udt_obj->getTotalTime(); //Dont minus dock/paid absence time. Because its already not included.
						$rate = NULL;
						//$pay_stub_entry = 'regular';
						$pay_stub_entry = $this->getPayStubEntryAccountLinkObject()->getRegularTime();

						unset($dock_absence_wage, $paid_absence_wage);
					} else {
						//Hourly
						Debug::text('Hourly or Hourly + Bonus Wage', __FILE__, __LINE__, __METHOD__,10);
						//$pay_stub->addEntry('regular', $this->getWageObject()->getWage( $udt_obj->getTotalTime(), $this->getWageObject()->getHourlyRate() ) , TTDate::getHours( $udt_obj->getTotalTime() ),  );
						//$pay_stub_entry = 'regular';
						$pay_stub_entry = $this->getPayStubEntryAccountLinkObject()->getRegularTime();
						$total_time = $udt_obj->getTotalTime();
						$amount = $this->getWage( $udt_obj->getTotalTime(), $this->getHourlyRate( $udt_obj->getColumn('user_wage_id') ) );
						$rate = $this->getHourlyRate( $udt_obj->getColumn('user_wage_id') );
					}
					Debug::text('aPay Stub Entry Account ID: '. $pay_stub_entry .' Amount: '. $amount, __FILE__, __LINE__, __METHOD__,10);
				} elseif ( $udt_obj->getStatus() == 10 AND $udt_obj->getType() == 30 ) { //Overtime
					Debug::text('User Total Row... Overtime Time: '. $udt_obj->getTotalTime() , __FILE__, __LINE__, __METHOD__,10);

					//Get overtime policy info.
					if ( $udt_obj->getOverTimePolicyObject()->getRate() > 0 ) {
						Debug::text('Paid Overtime Time Policy... Rate: '. $udt_obj->getOverTimePolicyObject()->getRate(), __FILE__, __LINE__, __METHOD__,10);
						//$pay_stub->addEntry('overtime_'.$udt_obj->getOverTimePolicyObject()->getLevel() , $this->getWageObject()->getWage( $udt_obj->getTotalTime(), ( $this->getWageObject()->getHourlyRate() * $udt_obj->getOverTimePolicyObject()->getRate() ) ) , TTDate::getHours( $udt_obj->getTotalTime() ), ( $this->getWageObject()->getHourlyRate() * $udt_obj->getOverTimePolicyObject()->getRate() ) );

						//$pay_stub_entry = 'overtime_'.$udt_obj->getOverTimePolicyObject()->getLevel();
						$pay_stub_entry = $udt_obj->getOverTimePolicyObject()->getPayStubEntryAccountId();
						$total_time = $udt_obj->getTotalTime();
						$amount = $this->getWage( $udt_obj->getTotalTime(), bcmul( $this->getHourlyRate( $udt_obj->getColumn('user_wage_id') ), $udt_obj->getOverTimePolicyObject()->getRate() ) );
						//$rate = ( $this->getHourlyRate( $udt_obj->getColumn('user_wage_id') ) * $udt_obj->getOverTimePolicyObject()->getRate() );
						$rate = bcmul( $this->getHourlyRate( $udt_obj->getColumn('user_wage_id') ), $udt_obj->getOverTimePolicyObject()->getRate() );
						Debug::text('bPay Stub Entry Account ID: '. $pay_stub_entry .' Amount: '. $amount, __FILE__, __LINE__, __METHOD__,10);
					} else {
						Debug::text('NOT Paid Overtime Time Policy: ', __FILE__, __LINE__, __METHOD__,10);
					}

				} elseif ( $udt_obj->getStatus() == 30 AND $udt_obj->getType() == 10) { //Absence
					Debug::text('User Total Row... Absence Time: '. $udt_obj->getTotalTime()  , __FILE__, __LINE__, __METHOD__,10);

					if ( is_object( $udt_obj->getAbsencePolicyObject() )
							AND $udt_obj->getAbsencePolicyObject()->getType() == 10
							AND $udt_obj->getAbsencePolicyObject()->getPayStubEntryAccountID() != '') { //Paid
						Debug::text('Paid Absence Time: '. $udt_obj->getTotalTime() , __FILE__, __LINE__, __METHOD__,10);

						//$pay_stub->addEntry( (int)$udt_obj->getAbsencePolicyObject()->getPayStubEntryNameID(), $this->getWage( $udt_obj->getTotalTime(), $this->getHourlyRate() ), TTDate::getHours( $udt_obj->getTotalTime() ), $this->getHourlyRate() );

						$pay_stub_entry = (int)$udt_obj->getAbsencePolicyObject()->getPayStubEntryAccountID();
						$total_time = $udt_obj->getTotalTime();
						$amount = $this->getWage( $udt_obj->getTotalTime(), $this->getHourlyRate( $udt_obj->getColumn('user_wage_id') ) );
						$rate = $this->getHourlyRate( $udt_obj->getColumn('user_wage_id') );
						//Debug::text('Paid Absence Info: '. $udt_obj->getTotalTime() , __FILE__, __LINE__, __METHOD__,10);
						Debug::text('cPay Stub Entry Account ID: '. $pay_stub_entry .' Amount: '. $amount, __FILE__, __LINE__, __METHOD__,10);

						//$paid_absence_time += $udt_obj->getTotalTime();
						$paid_absence_time = bcadd($paid_absence_time, $udt_obj->getTotalTime() );
						//$paid_absence_amount += $amount;
						$paid_absence_amount = bcadd( $paid_absence_amount, $amount);

						//Make sure we add the amount below. Incase there are two or more
						//entries for a paid absence in the same user_wage_id on one pay stub.
						if ( !isset($paid_absence_amount_arr[$udt_obj->getColumn('user_wage_id')]) ) {
							$paid_absence_amount_arr[$udt_obj->getColumn('user_wage_id')] = 0;
						}
						$paid_absence_amount_arr[$udt_obj->getColumn('user_wage_id')] = bcadd($paid_absence_amount_arr[$udt_obj->getColumn('user_wage_id')], $amount );

						//Sum all paid vacation amounts, so we don't calculate vacation on vacation
						//Handle this in Tax Deduction Exclude instead.
						/*
						if ( in_array( $udt_obj->getAbsencePolicyObject()->getPayStubEntryAccountID(), array($this->getPayStubEntryAccountLinkObject()->getVacationAccrual(), $this->getPayStubEntryAccountLinkObject()->getVacationAccrualRelease()) ) ) {
							Debug::text('Found Vacation Release... Pay Stub Entry ID: '. $udt_obj->getAbsencePolicyObject()->getPayStubEntryAccountID() , __FILE__, __LINE__, __METHOD__,10);
							//$paid_vacation_time += $udt_obj->getTotalTime();
							$paid_vacation_time = bcadd($paid_vacation_time, $udt_obj->getTotalTime() );
							//$paid_vacation_amount += $amount;
							$paid_vacation_amount = bcadd( $paid_vacation_amount, $amount );
						}
						*/
					} elseif ( is_object($udt_obj->getAbsencePolicyObject()) AND $udt_obj->getAbsencePolicyObject()->getType() == 30 ) {
						//$dock_absence_time += $udt_obj->getTotalTime();
						$dock_absence_time = bcadd( $dock_absence_time, $udt_obj->getTotalTime() );
						$amount = $this->getWage( $udt_obj->getTotalTime(), $this->getHourlyRate( $udt_obj->getColumn('user_wage_id') ) );
						//$dock_absence_amount += $amount;
						$dock_absence_amount = bcadd( $dock_absence_amount, $amount );
						$dock_absence_amount_arr[$udt_obj->getColumn('user_wage_id')] = $amount;
						Debug::text('DOCK Absence Time.. Adding: '. $udt_obj->getTotalTime() .' Total: '. $dock_absence_time , __FILE__, __LINE__, __METHOD__,10);
					}
				} elseif (  $udt_obj->getStatus() == 10 AND $udt_obj->getType() == 40 ) { //Premium
					Debug::text('User Total Row... Premium Time: '. $udt_obj->getTotalTime(), __FILE__, __LINE__, __METHOD__,10);

					//Get premium policy info.
					if ( $udt_obj->getPremiumPolicyObject()->getRate() > 0 ) {
						Debug::text('Paid Premium Time Policy... Rate: '. $udt_obj->getPremiumPolicyObject()->getRate(), __FILE__, __LINE__, __METHOD__,10);
						//$pay_stub->addEntry('overtime_'.$udt_obj->getOverTimePolicyObject()->getLevel() , $this->getWageObject()->getWage( $udt_obj->getTotalTime(), ( $this->getWageObject()->getHourlyRate() * $udt_obj->getOverTimePolicyObject()->getRate() ) ) , TTDate::getHours( $udt_obj->getTotalTime() ), ( $this->getWageObject()->getHourlyRate() * $udt_obj->getOverTimePolicyObject()->getRate() ) );

						switch ( $udt_obj->getPremiumPolicyObject()->getPayType() ) {
							case 10: //Pay Factor
								//Since they are already paid for this time with regular or OT, minus 1 from the rate
								//$rate = ( $this->getHourlyRate( $udt_obj->getColumn('user_wage_id') ) * ( $udt_obj->getPremiumPolicyObject()->getRate() - 1) );
								$rate = bcmul( $this->getHourlyRate( $udt_obj->getColumn('user_wage_id') ), bcsub( $udt_obj->getPremiumPolicyObject()->getRate(), 1 ) );
								break;
							case 20: //Pay Plus Premium
								$rate = $udt_obj->getPremiumPolicyObject()->getRate();
								break;
						}

						//$pay_stub_entry = 'premium_'.$udt_obj->getPremiumPolicyObject()->getLevel();
						$pay_stub_entry = $udt_obj->getPremiumPolicyObject()->getPayStubEntryAccountId();
						$total_time = $udt_obj->getTotalTime();
						$amount = $this->getWage( $udt_obj->getTotalTime(), $rate );
						Debug::text('dPay Stub Entry Account ID: '. $pay_stub_entry .' Amount: '. $amount, __FILE__, __LINE__, __METHOD__,10);
					} else {
						Debug::text('NOT Paid Premium Time Policy: ', __FILE__, __LINE__, __METHOD__,10);
					}

				}

				if ( isset($pay_stub_entry) AND $pay_stub_entry != '' ) {
					Debug::text('zPay Stub Entry Account ID: '. $pay_stub_entry .' Amount: '. $amount, __FILE__, __LINE__, __METHOD__,10);
					$ret_arr['entries'][] = array(	'user_wage_id' => $udt_obj->getColumn('user_wage_id'),
													'pay_stub_entry' => $pay_stub_entry,
													'total_time' =>  $total_time,
													'amount' => $amount,
													'rate' => $rate
												);
				}
				unset($pay_stub_entry, $amount, $total_time, $rate);
			}
		} else {
			Debug::text('NO UserDate Total entries found.', __FILE__, __LINE__, __METHOD__,10);
		}

		$ret_arr['other']['paid_absence_time'] = $paid_absence_time;
		$ret_arr['other']['paid_vacation_time'] = $paid_vacation_time;
		$ret_arr['other']['dock_absence_time'] = $dock_absence_time;

		$ret_arr['other']['paid_absence_amount'] = $paid_absence_amount;
		$ret_arr['other']['paid_vacation_amount'] = $paid_vacation_amount;
		$ret_arr['other']['dock_absence_amount'] = $dock_absence_amount;

		if ( isset($ret_arr) ) {
			Debug::Arr($ret_arr, 'UserDateTotal Array', __FILE__, __LINE__, __METHOD__,10);
			return $this->user_date_total_arr = $ret_arr;
		}

		return FALSE;
	}
}
?>