<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 430 $
 * $Id: HierarchyListFactory.class.php 430 2006-12-12 23:03:13Z ipso $
 * $Date: 2006-12-12 15:03:13 -0800 (Tue, 12 Dec 2006) $
 */
class HierarchyListFactory extends HierarchyFactory implements IteratorAggregate {

	protected $fasttree_obj = NULL;

	function getFastTreeObject() {

		if ( is_object($this->fasttree_obj) ) {
			return $this->fasttree_obj;
		} else {
			global $fast_tree_options;
			$this->fasttree_obj = new FastTree($fast_tree_options);

			return $this->fasttree_obj;
		}
	}

	function getChildLevelIdArrayByHierarchyControlIdAndUserId($tree_id, $user_id) {
		//This only gets the immediate children

		if ( $tree_id == '' ) {
			return FALSE;
		}

		if ( $user_id == '' ) {
			return FALSE;
		}

		$this->getFastTreeObject()->setTree( $tree_id );

		//Get current level IDs first, then get children of all of them.
		$ids = $this->getCurrentLevelIdArrayByHierarchyControlIdAndUserId($tree_id, $user_id);
		Debug::Arr($ids ,' Nodes at the same level', __FILE__, __LINE__, __METHOD__,10);

		if ( $ids === FALSE ) {
			return FALSE;
		}

		$hslf = new HierarchyShareListFactory();

		$retarr = array();
		foreach ( $ids as $id ) {
			Debug::Text(' Getting Children of ID: '. $id, __FILE__, __LINE__, __METHOD__,10);
			$children = $this->getFastTreeObject()->getAllChildren( $id );
			if ( $children === FALSE ) {
				continue;
			}

			$child_ids = array_keys( $children );

			$retarr = array_merge($retarr, $child_ids);
			unset($child_ids);
		}

		return $retarr;
	}

	function getAllParentLevelIdArrayByHierarchyControlIdAndUserId($tree_id, $user_id) {
		//This only gets the immediate parents
		if ( $tree_id == '' ) {
			return FALSE;
		}

		if ( $user_id == '' ) {
			return FALSE;
		}

		$this->getFastTreeObject()->setTree( $tree_id );
/*
		//Get the parent, then get the current level from that.
		$parent_id = $this->getFastTreeObject()->getParentId( $user_id );

		$retarr = $this->getCurrentLevelIdArrayByHierarchyControlIdAndUserId($tree_id, $parent_id);
		Debug::Arr($retarr ,' Parent Nodes', __FILE__, __LINE__, __METHOD__,10);
*/

		$ids = $this->getFastTreeObject()->getAllParents( $user_id );
		Debug::Arr($ids ,' Parent Nodes', __FILE__, __LINE__, __METHOD__,10);

		//Find out if any of the parents are shared.
		$hslf = new HierarchyShareListFactory();

		$retarr = array();
		foreach ( $ids as $id ) {

			$hierarchy_share = $hslf->getByHierarchyControlIdAndUserId( $tree_id, $id )->getCurrent()->isNew();

			if ( $hierarchy_share === FALSE ) {
				Debug::Text(' Node IS shared:  '. $id, __FILE__, __LINE__, __METHOD__,10);

				//Get current level IDs
				$current_level_ids = $this->getCurrentLevelIdArrayByHierarchyControlIdAndUserId( $tree_id, $id );
				$retarr = array_merge($retarr, $current_level_ids);
				unset($current_level_ids);
			} else {
				Debug::Text(' Node isnt shared:  '. $id, __FILE__, __LINE__, __METHOD__,10);
				$retarr[] = (int)$id;
			}
		}

		Debug::Arr($retarr ,' Final Parent Nodes including shared', __FILE__, __LINE__, __METHOD__,10);
		return array_unique($retarr);
	}


	function getParentLevelIdArrayByHierarchyControlIdAndUserId($tree_id, $user_id) {
		//This only gets the immediate parents
		if ( $tree_id == '' ) {
			return FALSE;
		}

		if ( $user_id == '' ) {
			return FALSE;
		}

		$this->getFastTreeObject()->setTree( $tree_id );

		//Get the parent, then get the current level from that.
		$parent_id = $this->getFastTreeObject()->getParentId( $user_id );

		$retarr = $this->getCurrentLevelIdArrayByHierarchyControlIdAndUserId($tree_id, $parent_id);
		Debug::Arr($retarr ,' Parent Nodes', __FILE__, __LINE__, __METHOD__,10);

		return $retarr;
	}

	function getCurrentLevelIdArrayByHierarchyControlIdAndUserId($tree_id, $user_id) {
		if ( $tree_id == '' ) {
			return FALSE;
		}

		if ( $user_id == '' ) {
			return FALSE;
		}

		$this->getFastTreeObject()->setTree( $tree_id );

		$parent_id = $this->getFastTreeObject()->getParentId( $user_id);

		$children = $this->getFastTreeObject()->getAllChildren( $parent_id );
		if ( $children === FALSE ) {
			return FALSE;
		}

		$ids = array_keys( $children );
		Debug::Arr($ids ,' Nodes at the same level', __FILE__, __LINE__, __METHOD__,10);

		$hslf = new HierarchyShareListFactory();

		$retarr[] = (int)$user_id;
		foreach ( $ids as $id ) {

			$hierarchy_share = $hslf->getByHierarchyControlIdAndUserId( $tree_id, $id )->getCurrent()->isNew();

			if ( $hierarchy_share === FALSE ) {
				Debug::Text(' Node IS shared:  '. $id, __FILE__, __LINE__, __METHOD__,10);
				$retarr[] = $id;

			} else {
				Debug::Text(' Node isnt shared:  '. $id, __FILE__, __LINE__, __METHOD__,10);
			}
		}

		return array_unique($retarr);
	}
/*
	function getLevelsByHierarchyControlIdAndUserId( $tree_id, $user_id ) {
		if ( $tree_id == '' ) {
			return FALSE;
		}

		if ( $user_id == '' ) {
			return FALSE;
		}

		$this->getFastTreeObject()->setTree( $tree_id );

		$node_level = $this->getFastTreeObject()->getLevel( $user_id );

		$children = $this->getFastTreeObject()->getAllChildren( $user_id, 'RECURSE' );
		if ( $children === FALSE ) {
			Debug::Text(' No children found, returning FALSE:  ', __FILE__, __LINE__, __METHOD__,10);
			return FALSE;
		}
		Debug::Text(' Children FOUND, NOT returning FALSE:  ', __FILE__, __LINE__, __METHOD__,10);
		Debug::Arr($children ,' Children', __FILE__, __LINE__, __METHOD__,10);
		//$levels = array_unique( array_values( $children ) );
		$levels = array_unique( $children );

		foreach( $levels as $key => $value ) {
			$retarr[$key] = $value + $node_level;
		}
		Debug::Arr($retarr ,' RetArr', __FILE__, __LINE__, __METHOD__,10);

		//Ignore the highest level.
		asort($retarr);
		array_pop($retarr);

		$retarr = array_flip($retarr);

		//var_dump($retarr);

		return $retarr;
	}
*/

	function getLevelsByHierarchyControlIdAndUserId( $tree_id, $user_id ) {
		if ( $tree_id == '' ) {
			return FALSE;
		}

		if ( $user_id == '' ) {
			return FALSE;
		}

		$this->getFastTreeObject()->setTree( $tree_id );

		$node_level = $this->getFastTreeObject()->getLevel( $user_id );
		Debug::Text(' Node Level: '. $node_level, __FILE__, __LINE__, __METHOD__,10);

		$current_level_nodes = $this->getCurrentLevelIdArrayByHierarchyControlIdAndUserId($tree_id, $user_id);
		Debug::Arr($current_level_nodes, ' Current Level Nodes  ', __FILE__, __LINE__, __METHOD__,10);

		//Do this because of shared hierarchy levels. Only one user can have unique children, so we
		//need to go through each user at the same level and collect all children.
		if ( count($current_level_nodes) > 0 ) {

			foreach($current_level_nodes as $tmp_user_id ) {
				Debug::Text(' Getting Children for tmp user ID: '. $tmp_user_id, __FILE__, __LINE__, __METHOD__,10);

				$children = $this->getFastTreeObject()->getAllChildren( $tmp_user_id, 'RECURSE' );

				if ( $children === FALSE ) {
					Debug::Text(' No children found, returning FALSE:  ', __FILE__, __LINE__, __METHOD__,10);
					//return FALSE;
					continue;
				}

				Debug::Text(' Children FOUND, NOT returning FALSE:  ', __FILE__, __LINE__, __METHOD__,10);
				Debug::Arr($children ,' Children', __FILE__, __LINE__, __METHOD__,10);
				//$levels = array_unique( array_values( $children ) );
				$levels = array_unique( $children );

				foreach( $levels as $key => $value ) {
					$retarr[$key] = $value + $node_level;
				}

				Debug::Arr($retarr ,' RetArr', __FILE__, __LINE__, __METHOD__,10);
			}

			if ( isset($retarr) ) {
				//Ignore the highest level.
				asort($retarr);
				array_pop($retarr);

				$retarr = array_flip($retarr);
				//$retarr = array_merge($retarr, $tmp_retarr);

				return $retarr;
			}
		}


		//var_dump($retarr);
		return FALSE;
		//return $retarr;
	}

	function getByHierarchyControlIdAndUserId( $tree_id, $user_id ) {
		if ( $tree_id == '' ) {
			return FALSE;
		}

		if ( $user_id == '' ) {
			return FALSE;
		}

		$this->getFastTreeObject()->setTree( $tree_id );

		$node = $this->getFastTreeObject()->getNode( $user_id );

		if ($node === FALSE ) {
			return FALSE;
		}

		$ulf = new UserListFactory();
		$user_obj = $ulf->getById( $node['object_id'] )->getCurrent();

		$hslf = new HierarchyShareListFactory();
		$hierarchy_share = $hslf->getByHierarchyControlIdAndUserId( $tree_id, $user_id )->getCurrent()->isNew();

		if ( $hierarchy_share === FALSE ) {
			$shared = TRUE;
		} else {
			$shared = FALSE;
		}

		$retarr = array(
						'id' => $node['object_id'],
						'parent_id' => $node['parent_id'],
						'name' => $user_obj->getFullName(),
						'level' => $node['level'],
						'shared' => $shared

						);

		return $retarr;
	}

	function getByCompanyIdAndHierarchyControlId( $company_id, $tree_id ) {
		if ( $company_id == '' ) {
			return FALSE;
		}

		if ( $tree_id == '' ) {
			return FALSE;
		}

		$hclf = new HierarchyControlListFactory();
		$hclf->getByIdAndCompanyId($tree_id, $company_id);
		if ( $hclf->getRecordCount() == 0 ) {
			return FALSE;
		}

		return $this->getByHierarchyControlId( $tree_id );
	}

	function getByHierarchyControlId( $tree_id ) {
		if ( $tree_id == '' ) {
			return FALSE;
		}

		$this->getFastTreeObject()->setTree( $tree_id );

		$children = $this->getFastTreeObject()->getAllChildren(NULL, 'RECURSE');

		$ulf = new UserListFactory();
		$hslf = new HierarchyShareListFactory();
		$hslf->getByHierarchyControlId( $tree_id );
		$shared_user_ids = array();
		foreach( $hslf as $hierarchy_share ) {
			$shared_user_ids[] = $hierarchy_share->getUser();
		}

		if ( $children !== FALSE ) {
			foreach ($children as $object_id => $level ) {

				if ( $object_id !== 0 ) {
					$user_obj = $ulf->getById ( $object_id )->getCurrent();

					unset($shared);
					if ( in_array( $object_id, $shared_user_ids) === TRUE ) {
						$shared = TRUE;
					} else {
						$shared = FALSE;
					}

					$nodes[] = array(
									'id' => $object_id,
									'name' => $user_obj->getFullName(),
									'level' => $level,
									'shared' => $shared
									);
				}

			}

			if ( isset($nodes) ) {
				return $nodes;
			}
		}

		return FALSE;

	}

	function getHierarchyChildrenByCompanyIdAndUserIdAndObjectTypeID( $company_id, $user_id,  $object_type_id = 100 ) {
		global $profiler;
		$profiler->startTimer( "getPermissionHierarchyChildrenByCompanyIdAndUserId" );

		if ( $company_id == '' ) {
			return FALSE;
		}

		if ( $user_id == '' ) {
			return FALSE;
		}

		if ( $object_type_id == '' ) {
			return FALSE;
		}

		$hotlf = new HierarchyObjectTypeListFactory();
		$hotlf->getByCompanyIdAndObjectTypeId($company_id, $object_type_id );
		if ( $hotlf->getRecordCount() > 0 ) {
			$hierarchy_control_id = $hotlf->getCurrent()->getHierarchyControl();
		} else {
			//No Permission hierarhcy.
			$profiler->stopTimer( "getPermissionHierarchyChildrenByCompanyIdAndUserId" );
			return FALSE;
		}
		Debug::Text(' Hierarchy Control ID: '. $hierarchy_control_id , __FILE__, __LINE__, __METHOD__,10);

		$cache_id = $hierarchy_control_id.$user_id;

		$retval = $this->getCache($cache_id);
		if ( $retval === FALSE ) {
			$hlf = new HierarchyListFactory();
			$retval = $hlf->getChildLevelIdArrayByHierarchyControlIdAndUserId( $hierarchy_control_id, $user_id );

			$this->saveCache($retval,$cache_id);
		}

		$profiler->stopTimer( "getPermissionHierarchyChildrenByCompanyIdAndUserId" );
		return $retval;
	}
}
?>
