<?php
/*********************************************************************************
 * The contents of this file are subject to the TimeTrex Public License Version
 * 1.1.0 ("License"); You may not use this file except in compliance with the
 * License. You may obtain a copy of the License at http://www.TimeTrex.com/TPL
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied.  See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * All copies of the Covered Code must include on each user interface screen:
 *    (i) the "Powered by TimeTrex" logo and
 *    (ii) the TimeTrex copyright notice
 * in the same form as they appear in the distribution.  See full license for
 * requirements.
 *
 * The Original Code is: TimeTrex Open Source
 * The Initial Developer of the Original Code is TimeTrex Payroll Services
 * Portions created by TimeTrex are Copyright (C) 2004-2007 TimeTrex Payroll Services;
 * All Rights Reserved.
 *
 ********************************************************************************/
/*
 * $Revision: 417 $
 * $Id: PunchControlFactory.class.php 417 2006-12-06 22:58:53Z ipso $
 * $Date: 2006-12-06 14:58:53 -0800 (Wed, 06 Dec 2006) $
 */
class PunchControlFactory extends Factory {
	protected $table = 'punch_control';
	protected $pk_sequence_name = 'punch_control_id_seq'; //PK Sequence name

	protected $tmp_data = NULL;

	protected $user_date_obj = NULL;
	protected $job_obj = NULL;
	protected $meal_policy_obj = NULL;

	protected $userdate_validator_msg = 'Invalid User Date ID';
	protected $userdate_validator_label = 'user_date';

	protected $branch_validator_msg = 'Branch does not exist';
	protected $branch_validator_label = 'branch';

	protected $department_validator_msg = 'Department does not exist';
	protected $department_validator_label = 'department';

	protected $job_validator_msg = 'Job does not exist';
	protected $job_validator_label = 'job';

	protected $jobitem_validator_msg = 'Job Item does not exist';
	protected $jobitem_validator_label = 'job_item';

	protected $quantity_validator_isfloat_msg = 'Incorrect quantity';
	protected $quantity_validator_label = 'quantity';

	protected $badquantity_validator_isfloat_msg = 'Incorrect bad quantity';
	protected $badquantity_validator_label = 'bad_quantity';

	protected $totaltime_validator_isnumeric_msg = 'Incorrect total time';
	protected $totaltime_validator_label = 'total_time';

	protected $actualtotaltime_validator_isnumeric_msg = 'Incorrect actual total time';
	protected $actualtotaltime_validator_label = 'actual_total_time';

	protected $mealpolicy_validator_msg = 'Meal Policy is invalid';
	protected $mealpolicy_validator_label = 'meal_policy';

	protected $payperiod_validator_locked_msg = 'Pay Period is Currently Locked, or Date/Time is incorrect';
	protected $payperiod_validator_label = 'pay_period';

	protected $note_validator_length_msg = 'Note is too short or too long';
	protected $note_validator_label = 'note';

	protected $otherid1_validator_islength_msg = 'Other ID 1 is invalid';
	protected $otherid1_validator_label = 'other_id1';

	protected $otherid2_validator_islength_msg = 'Other ID 2 is invalid';
	protected $otherid2_validator_label = 'other_id2';

	protected $otherid3_validator_islength_msg = 'Other ID 3 is invalid';
	protected $otherid3_validator_label = 'other_id3';

	protected $otherid4_validator_islength_msg = 'Other ID 4 is invalid';
	protected $otherid4_validator_label = 'other_id4';

	protected $otherid5_validator_islength_msg = 'Other ID 5 is invalid';
	protected $otherid5_validator_label = 'other_id5';

	function getUserDateObject() {
		if ( is_object($this->user_date_obj) ) {
			return $this->user_date_obj;
		} else {
			$udlf = new UserDateListFactory();
			$udlf->getById( $this->getUserDateID() );
			if ( $udlf->getRecordCount() > 0 ) {
				$this->user_date_obj = $udlf->getCurrent();
				return $this->user_date_obj;
			}

			return FALSE;
		}
	}

	function getJobObject() {
		if ( is_object($this->job_obj) ) {
			return $this->job_obj;
		} else {
			$jlf = new JobListFactory();
			$jlf->getById( $this->getJob() );
			if ( $jlf->getRecordCount() > 0 ) {
				$this->job_obj = $jlf->getCurrent();
				return $this->job_obj;
			}

			return FALSE;
		}
	}

	function findUserDate($user_id, $epoch, $status_id = 10, $punch_id = 0 ) {
		//Get pay period start/continuous time
		$ppslf = new PayPeriodScheduleListFactory();
		$ppslf->getByUserId( $user_id );
		if ( $ppslf->getRecordCount() == 1 ) {
			$pps_obj = $ppslf->getCurrent();
			Debug::Text(' Pay Period Schedule Continuous Time: '. $pps_obj->getContinuousTime(), __FILE__, __LINE__, __METHOD__,10);

			//If this is an out punch, we need to find the timestamp of the In punch to base
			//continuous time off of.
			$plf = new PunchListFactory();
			if ( $status_id == 20 ) {
				//getPreviousPunchByUserIdAndEpoch uses <= $epoch, so we minus one second from it
				//so it doesn't find the punch we may be editing.
				$plf->getPreviousPunchByUserIdAndEpochAndNotPunchIDAndContinuousTime( $user_id, $epoch-1, $punch_id, 86400 );
				if ( $plf->getRecordCount() > 0 ) {
					$p_obj = $plf->getCurrent();
					$epoch = $p_obj->getTimeStamp();
					Debug::Text('Found In Punch Epoch: '. $epoch .' - '. TTDate::getDate('DATE+TIME', $epoch) , __FILE__, __LINE__, __METHOD__,10);
					unset($p_obj);
				} else {
					Debug::Text('DID NOT Find In Punch Epoch: '. $epoch, __FILE__, __LINE__, __METHOD__,10);
				}
			}

			$plf->getFirstPunchByUserIDAndEpoch( $user_id, $epoch, $pps_obj->getContinuousTime() );
			if ( $plf->getRecordCount() > 0 ) {
				$p_obj = $plf->getCurrent();
				if ( ( $epoch - $p_obj->getTimeStamp() ) <= $pps_obj->getContinuousTime() ) {
					$user_date_id = $p_obj->getPunchControlObject()->getUserDateID();
					Debug::text(' User Date ID found: '. $user_date_id, __FILE__, __LINE__, __METHOD__,10);

					$this->setUserDateID( $user_date_id );
					return TRUE;
				}
			}
		}
		Debug::Text('Skipping Find...', __FILE__, __LINE__, __METHOD__,10);
		Debug::Text(' Pay Period Schedule Record Count: '.$ppslf->getRecordCount() , __FILE__, __LINE__, __METHOD__,10);

		return $this->setUserDate( $user_id, $epoch );
	}

	function setUserDate($user_id, $date) {
		$user_date_id = UserDateFactory::findOrInsertUserDate( $user_id, $date);
		Debug::text(' User Date ID: '. $user_date_id, __FILE__, __LINE__, __METHOD__,10);
		if ( $user_date_id != '' ) {
			$this->setUserDateID( $user_date_id );
			return TRUE;
		}
		Debug::text(' No User Date ID found', __FILE__, __LINE__, __METHOD__,10);

		return FALSE;
	}

	function getOldUserDateID() {
		if ( isset($this->tmp_data['old_user_date_id']) ) {
			return $this->tmp_data['old_user_date_id'];
		}

		return FALSE;
	}
	function setOldUserDateID($id) {
		Debug::Text(' Setting Old User Date ID: '. $id, __FILE__, __LINE__, __METHOD__,10);
		$this->tmp_data['old_user_date_id'] = $id;

		return TRUE;
	}


	function getUserDateID() {
		if ( isset($this->data['user_date_id']) ) {
			return $this->data['user_date_id'];
		}

		return FALSE;
	}

	function setUserDateID($id = NULL) {
		$id = trim($id);

		$udlf = new UserDateListFactory();

		if (  $this->Validator->isResultSetWithRows(	$this->userdate_validator_label,
														$udlf->getByID($id),
														$this->userdate_validator_msg
														) ) {
			$this->data['user_date_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getBranch() {
		if ( isset($this->data['branch_id']) ) {
			return $this->data['branch_id'];
		}

		return FALSE;
	}
	function setBranch($id) {
		$id = trim($id);

		if ( $id == FALSE OR $id == 0 OR $id == '' ) {
			$id = 0;
		}

		$blf = new BranchListFactory();

		if (  $id == 0
				OR
				$this->Validator->isResultSetWithRows(	$this->branch_validator_label,
														$blf->getByID($id),
														$this->branch_validator_msg
														) ) {
			$this->data['branch_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getDepartment() {
		if ( isset($this->data['department_id']) ) {
			return $this->data['department_id'];
		}

		return FALSE;
	}
	function setDepartment($id) {
		$id = trim($id);

		if ( $id == FALSE OR $id == 0 OR $id == '' ) {
			$id = 0;
		}

		$dlf = new DepartmentListFactory();

		if (  $id == 0
				OR
				$this->Validator->isResultSetWithRows(	$this->department_validator_label,
														$dlf->getByID($id),
														$this->department_validator_msg
														) ) {
			$this->data['department_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getJob() {
		if ( isset($this->data['job_id']) ) {
			return $this->data['job_id'];
		}

		return FALSE;
	}
	function setJob($id) {
		$id = trim($id);

		if ( $id == FALSE OR $id == 0 OR $id == '' ) {
			$id = 0;
		}

		if ( isProfessionalEditionAvailable() == TRUE ) {
			$jlf = new JobListFactory();
		}

		if (  $id == 0
				OR
				$this->Validator->isResultSetWithRows(	$this->job_validator_label,
														$jlf->getByID($id),
														$this->job_validator_msg
														) ) {
			$this->data['job_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getJobItem() {
		if ( isset($this->data['job_item_id']) ) {
			return $this->data['job_item_id'];
		}

		return FALSE;
	}
	function setJobItem($id) {
		$id = trim($id);

		if ( $id == FALSE OR $id == 0 OR $id == '' ) {
			$id = 0;
		}

		if ( isProfessionalEditionAvailable() == TRUE ) {
			$jilf = new JobItemListFactory();
		}

		if (  $id == 0
				OR
				$this->Validator->isResultSetWithRows(	$this->jobitem_validator_label,
														$jilf->getByID($id),
														$this->jobitem_validator_msg
														) ) {
			$this->data['job_item_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getQuantity() {
		if ( isset($this->data['quantity']) ) {
			return (float)$this->data['quantity'];
		}

		return FALSE;
	}
	function setQuantity($val) {
		$val = (float)$val;

		if ( $val == FALSE OR $val == 0 OR $val == '' ) {
			$val = 0;
		}

		if 	(	$val == 0
				OR
				$this->Validator->isFloat(			$this->quantity_validator_label,
													$val,
													$this->quantity_validator_isfloat_msg) ) {
			$this->data['quantity'] = $val;

			return TRUE;
		}

		return FALSE;
	}

	function getBadQuantity() {
		if ( isset($this->data['bad_quantity']) ) {
			return (float)$this->data['bad_quantity'];
		}

		return FALSE;
	}
	function setBadQuantity($val) {
		$val = (float)$val;

		if ( $val == FALSE OR $val == 0 OR $val == '' ) {
			$val = 0;
		}

		if 	(	$val == 0
				OR
				$this->Validator->isFloat(			$this->badquantity_validator_label,
													$val,
													$this->badquantity_validator_isfloat_msg) ) {
			$this->data['bad_quantity'] = $val;

			return TRUE;
		}

		return FALSE;
	}

	function getTotalTime() {
		if ( isset($this->data['total_time']) ) {
			return (int)$this->data['total_time'];
		}
		return FALSE;
	}
	function setTotalTime($int) {
		$int = (int)$int;

		if 	(	$this->Validator->isNumeric(		$this->totaltime_validator_label,
													$int,
													$this->totaltime_validator_isnumeric_msg) ) {
			$this->data['total_time'] = $int;

			return TRUE;
		}

		return FALSE;
	}

	function getActualTotalTime() {
		if ( isset($this->data['actual_total_time']) ) {
			return (int)$this->data['actual_total_time'];
		}
		return FALSE;
	}
	function setActualTotalTime($int) {
		$int = (int)$int;

		if ( $int < 0 ) {
			$int = 0;
		}

		if 	(	$this->Validator->isNumeric(		$this->actualtotaltime_validator_label,
													$int,
													$this->actualtotaltime_validator_isnumeric_msg) ) {
			$this->data['actual_total_time'] = $int;

			return TRUE;
		}

		return FALSE;
	}

	function getMealPolicyID() {
		if ( isset($this->data['meal_policy_id']) ) {
			return $this->data['meal_policy_id'];
		}

		return FALSE;
	}
	function setMealPolicyID($id) {
		$id = trim($id);

		if ( $id == '' OR empty($id) ) {
			$id = NULL;
		}

		$mplf = new MealPolicyListFactory();

		if ( $id == NULL
				OR
				$this->Validator->isResultSetWithRows(	$this->mealpolicy_validator_label,
														$mplf->getByID($id),
														$this->mealpolicy_validator_msg
													) ) {

			$this->data['meal_policy_id'] = $id;

			return TRUE;
		}

		return FALSE;
	}

	function getNote() {
		if ( isset($this->data['note']) ) {
			return $this->data['note'];
		}
	}
	function setNote($val) {
		$val = trim($val);

		if 	(	$val == ''
				OR
				$this->Validator->isLength(		$this->note_validator_label,
												$val,
												$this->note_validator_length_msg,
												0,
												1024) ) {

			$this->data['note'] = $val;

			return TRUE;
		}

		return FALSE;
	}

	function getOtherID1() {
		if ( isset($this->data['other_id1']) ) {
			return $this->data['other_id1'];
		}

		return FALSE;
	}
	function setOtherID1($value) {
		$value = trim($value);

		if (	$value == ''
				OR
				$this->Validator->isLength(	$this->otherid1_validator_label,
											$value,
											$this->otherid1_validator_islength_msg,
											1,255) ) {

			$this->data['other_id1'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getOtherID2() {
		if ( isset($this->data['other_id2']) ) {
			return $this->data['other_id2'];
		}

		return FALSE;
	}
	function setOtherID2($value) {
		$value = trim($value);

		if (	$value == ''
				OR
				$this->Validator->isLength(	$this->otherid2_validator_label,
											$value,
											$this->otherid2_validator_islength_msg,
											1,255) ) {

			$this->data['other_id2'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getOtherID3() {
		if ( isset($this->data['other_id3']) ) {
			return $this->data['other_id3'];
		}

		return FALSE;
	}
	function setOtherID3($value) {
		$value = trim($value);

		if (	$value == ''
				OR
				$this->Validator->isLength(	$this->otherid3_validator_label,
											$value,
											$this->otherid3_validator_islength_msg,
											1,255) ) {

			$this->data['other_id3'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getOtherID4() {
		if ( isset($this->data['other_id4']) ) {
			return $this->data['other_id4'];
		}

		return FALSE;
	}
	function setOtherID4($value) {
		$value = trim($value);

		if (	$value == ''
				OR
				$this->Validator->isLength(	$this->otherid4_validator_label,
											$value,
											$this->otherid4_validator_islength_msg,
											1,255) ) {

			$this->data['other_id4'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function getOtherID5() {
		if ( isset($this->data['other_id5']) ) {
			return $this->data['other_id5'];
		}

		return FALSE;
	}
	function setOtherID5($value) {
		$value = trim($value);

		if (	$value == ''
				OR
				$this->Validator->isLength(	$this->otherid5_validator_label,
											$value,
											$this->otherid5_validator_islength_msg,
											1,255) ) {

			$this->data['other_id5'] = $value;

			return TRUE;
		}

		return FALSE;
	}

	function calcAutoDeductMealPolicyTime( $total_time, $schedule_obj) {
		//FIXME: Meal AutoDeducts only work for a single In/Out pair greater then
		//the trigger amount.
		//If users are transferring branches/departs, autodeducts are no longer working.
		//But if they are tranferring, why just punch out for lunch?
		$amount = 0;
		$meal_policy_id = NULL;
		Debug::text('Total Time: '. $total_time, __FILE__, __LINE__, __METHOD__, 10);

		if ( is_object($schedule_obj) AND is_object($schedule_obj->getSchedulePolicyObject() ) ) {
			Debug::text('Passed Schedule Obj: ', __FILE__, __LINE__, __METHOD__, 10);
			if ( is_object($schedule_obj->getSchedulePolicyObject()->getMealPolicyObject() )
					AND $schedule_obj->getSchedulePolicyObject()->getMealPolicyObject()->getType() == 10 ) {
				Debug::text('Found Scheduled Auto-Deduct Meal Policy: '. $schedule_obj->getSchedulePolicyObject()->getMealPolicyObject()->getId(), __FILE__, __LINE__, __METHOD__, 10);

				if ( $total_time > $schedule_obj->getSchedulePolicyObject()->getMealPolicyObject()->getTriggerTime() ) {
					$meal_policy_id = $schedule_obj->getSchedulePolicyObject()->getMealPolicyObject()->getId();
					$amount = $schedule_obj->getSchedulePolicyObject()->getMealPolicyObject()->getAmount();
				}
			}
		} else {
			Debug::text('NOT Passed Schedule Obj: ', __FILE__, __LINE__, __METHOD__, 10);
			$mplf = new MealPolicyListFactory();
			$mplf->getByPolicyGroupUserId( $this->getUserDateObject()->getUser() );
			if ( $mplf->getRecordCount() == 1 ) {
				$mp_obj = $mplf->getCurrent();
				Debug::text('Found Meal Policy: '. $mp_obj->getId(), __FILE__, __LINE__, __METHOD__, 10);

				if ( $mp_obj->getType() == 10 ) {
					Debug::text('Found Auto-Deduct Meal Policy: '. $mp_obj->getId(), __FILE__, __LINE__, __METHOD__, 10);

					if ( $total_time > $mp_obj->getTriggerTime() ) {
						$meal_policy_id = $mp_obj->getId();
						$amount = $mp_obj->getAmount();
					}
				}
			}
		}

		Debug::text('AutoDeduct Meal Policy Amount: '. $amount, __FILE__, __LINE__, __METHOD__, 10);
		return array('amount' => $amount, 'meal_policy_id' => $meal_policy_id);

	}

	function calcTotalTime() {
		$plf = new PunchListFactory();
		$plf->getByPunchControlId( $this->getId() );

		//Make sure punches are in In/Out pairs before we bother calculating.
		if ( ( $plf->getRecordCount() % 2 ) == 0 ) {
			Debug::text(' Found Punches to calculate.', __FILE__, __LINE__, __METHOD__,10);
			//$prev_status = NULL;
			$in_pair = FALSE;
			$schedule_obj = NULL;
			foreach( $plf as $punch_obj ) {
				//Check for proper in/out pairs
				//First row should be an Out status (reverse ordering)
				Debug::text(' Punch: Status: '. $punch_obj->getStatus() .' TimeStamp: '. $punch_obj->getTimeStamp(), __FILE__, __LINE__, __METHOD__,10);
				if ( $punch_obj->getStatus() == 20 ) {
					Debug::text(' Found Out Status, starting pair: ', __FILE__, __LINE__, __METHOD__,10);
					$out_stamp = $punch_obj->getTimeStamp();
					$out_actual_stamp = $punch_obj->getActualTimeStamp();
					$in_pair = TRUE;
				} elseif ( $in_pair == TRUE ) {
					$punch_obj->setScheduleId(); //Find Schedule Object for this Punch
					$schedule_obj = $punch_obj->getScheduleObject();
					$in_stamp = $punch_obj->getTimeStamp();
					$in_actual_stamp = $punch_obj->getActualTimeStamp();
					//Got a pair... Totaling.
					Debug::text(' Found a pair... Totaling: ', __FILE__, __LINE__, __METHOD__,10);
					if ( $out_stamp != '' AND $in_stamp != '' ) {
						$total_time = $out_stamp - $in_stamp;
					}
					if ( $out_actual_stamp != '' AND $in_actual_stamp != '' ) {
						$actual_total_time = $out_actual_stamp - $in_actual_stamp;
					}
				}
			}

			if ( isset($total_time) ) {
				Debug::text(' Setting TotalTime...', __FILE__, __LINE__, __METHOD__,10);
				//Check for AutoDeduct Meal Policy
				$meal_auto_deduct_arr = $this->calcAutoDeductMealPolicyTime( $total_time, $schedule_obj );

				$this->setTotalTime( $total_time - $meal_auto_deduct_arr['amount'] );
				$this->setActualTotalTime( $actual_total_time - $meal_auto_deduct_arr['amount'] );
				$this->setMealPolicyID( $meal_auto_deduct_arr['meal_policy_id'] );

				return TRUE;
			}
		} else {
			Debug::text(' No Punches to calculate, or punches arent in pairs. Set total to 0', __FILE__, __LINE__, __METHOD__,10);
			$this->setTotalTime( 0 );
			$this->setActualTotalTime( 0 );

			return TRUE;
		}

		return FALSE;
	}

/*
	function calcUserDateTotalTime() {

		//Add a row to the user date total table, as "worked" hours.
		//Edit if it already exists and is not set as override.
		$udtlf = new UserDateTotalListFactory();
		$udtlf->getByUserDateIdAndPunchControlId( $this->getUserDateID(), $this->getId() );
		if ( $udtlf->getRecordCount() > 0 ) {
			Debug::text(' Found Conflicting User Date Total Records, removing them before re-calc', __FILE__, __LINE__, __METHOD__,10);
			foreach($udtlf as $udt_obj) {
				$udt_obj->setDeleted(TRUE);
				$udt_obj->Save();
			}
		}

		$udtf = new UserDateTotalFactory();
		$udtf->setUserDateID( $this->getUserDateID() );
		$udtf->setPunchControlID( $this->getId() );
		$udtf->setStatus( 20 ); //Worked
		$udtf->setType( 10 ); //Total
		$udtf->setBranch( $this->getBranch() );
		$udtf->setDepartment( $this->getDepartment() );
		$udtf->setTotalTime( $this->getTotalTime() );
		$udtf->setEnableCalcSystemTotalTime( TRUE );
		return $udtf->Save();

		return TRUE;
	}
*/

	function getEnableCalcSystemTotalTime() {
		if ( isset($this->calc_system_total_time) ) {
			return $this->calc_system_total_time;
		}

		return FALSE;
	}
	function setEnableCalcSystemTotalTime($bool) {
		$this->calc_system_total_time = $bool;

		return TRUE;
	}

	function getEnableCalcWeeklySystemTotalTime() {
		if ( isset($this->calc_weekly_system_total_time) ) {
			return $this->calc_weekly_system_total_time;
		}

		return FALSE;
	}
	function setEnableCalcWeeklySystemTotalTime($bool) {
		$this->calc_weekly_system_total_time = $bool;

		return TRUE;
	}

	function getEnableCalcException() {
		if ( isset($this->calc_exception) ) {
			return $this->calc_exception;
		}

		return FALSE;
	}
	function setEnableCalcException($bool) {
		$this->calc_exception = $bool;

		return TRUE;
	}

	function getEnableCalcUserDateTotal() {
		if ( isset($this->calc_user_date_total) ) {
			return $this->calc_user_date_total;
		}

		return FALSE;
	}
	function setEnableCalcUserDateTotal($bool) {
		$this->calc_user_date_total = $bool;

		return TRUE;
	}

	function getEnableCalcTotalTime() {
		if ( isset($this->calc_total_time) ) {
			return $this->calc_total_time;
		}

		return FALSE;
	}
	function setEnableCalcTotalTime($bool) {
		$this->calc_total_time = $bool;

		return TRUE;
	}

	function Validate() {
		Debug::text('Validating...', __FILE__, __LINE__, __METHOD__,10);
		Debug::text('User Date Id: '. $this->getUserDateID(), __FILE__, __LINE__, __METHOD__,10);
		if ( $this->getUserDateObject() == FALSE
				OR $this->getUserDateObject()->getPayPeriodObject() == FALSE
				OR $this->getUserDateObject()->getPayPeriodObject()->getIsLocked() == TRUE ) {

				$this->Validator->isTRUE(	$this->payperiod_validator_label,
											FALSE,
											$this->payperiod_validator_locked_msg );

		}

		return TRUE;
	}

	function preSave() {
		if ( $this->getBranch() === FALSE ) {
			$this->setBranch(0);
		}

		if ( $this->getDepartment() === FALSE ) {
			$this->setDepartment(0);
		}

		if ( $this->getJob() === FALSE ) {
			$this->setJob(0);
		}

		if ( $this->getJobItem() === FALSE ) {
			$this->setJobItem(0);
		}

		if ( $this->getQuantity() === FALSE ) {
			$this->setQuantity(0);
		}

		if ( $this->getBadQuantity() === FALSE ) {
			$this->setBadQuantity(0);
		}

		//Set Job default Job Item if required.
		if ( $this->getJob() != FALSE AND $this->getJobItem() == '' ) {
			Debug::text(' Job is set, but no task is... Using default job item...', __FILE__, __LINE__, __METHOD__,10);

			if ( is_object( $this->getJobObject() ) ){
				Debug::text(' Default Job Item: '. $this->getJobObject()->getDefaultItem(), __FILE__, __LINE__, __METHOD__,10);
				$this->setJobItem( $this->getJobObject()->getDefaultItem() );
			}
		}

		if ( $this->getEnableCalcTotalTime() == TRUE ) {
			$this->calcTotalTime();
		}

		return TRUE;
	}

	function postSave() {
		Debug::text(' postSave()', __FILE__, __LINE__, __METHOD__,10);

		//Do this regardless of if EnableCalcUserDateTotal is set.
		Debug::text(' Old User Date ID: '. $this->getOldUserDateId() .' RAW: '. $this->tmp_data['old_user_date_id'], __FILE__, __LINE__, __METHOD__,10);
		if ( $this->getOldUserDateId() !== FALSE ) {
			Debug::text(' Recalculating Old User Date ID: '. $this->getOldUserDateId(), __FILE__, __LINE__, __METHOD__,10);
			UserDateTotalFactory::reCalculateDay( $this->getOldUserDateId(), TRUE );
		}

		if ( $this->getEnableCalcUserDateTotal() == TRUE ) {
			//Add a row to the user date total table, as "worked" hours.
			//Edit if it already exists and is not set as override.
			$udtlf = new UserDateTotalListFactory();
			$udtlf->getByUserDateIdAndPunchControlId( $this->getUserDateID(), $this->getId() );
			if ( $udtlf->getRecordCount() > 0 ) {
				Debug::text(' Found Conflicting User Date Total Records, removing them before re-calc', __FILE__, __LINE__, __METHOD__,10);
				foreach($udtlf as $udt_obj) {
					//$udt_obj->setDeleted(TRUE);
					//$udt_obj->Save();
					if ( $udt_obj->getOverride() == FALSE ) {
						$udt_obj->Delete();
					}
				}
			}

			if ( $this->getDeleted() == FALSE ) {
				Debug::text(' Calculating Total Time for day.', __FILE__, __LINE__, __METHOD__,10);
				$udtf = new UserDateTotalFactory();
				$udtf->setUserDateID( $this->getUserDateID() );
				$udtf->setPunchControlID( $this->getId() );
				$udtf->setStatus( 20 ); //Worked
				$udtf->setType( 10 ); //Total

				$udtf->setBranch( $this->getBranch() );
				$udtf->setDepartment( $this->getDepartment() );

				$udtf->setJob( $this->getJob() );
				$udtf->setJobItem( $this->getJobItem() );
				$udtf->setQuantity( $this->getQuantity() );
				$udtf->setBadQuantity( $this->getBadQuantity() );

				$udtf->setTotalTime( $this->getTotalTime() );
				$udtf->setActualTotalTime( $this->getActualTotalTime() );

				$udtf->setEnableCalcSystemTotalTime( $this->getEnableCalcSystemTotalTime() );
				$udtf->setEnableCalcWeeklySystemTotalTime( $this->getEnableCalcWeeklySystemTotalTime() );
				$udtf->setEnableCalcException( $this->getEnableCalcException() );
				if ( $udtf->isValid() ) {
					$udtf->Save();
				}

				//Only calc exceptions when calculating system total time?
				//We have a flag to enable them..
				//ExceptionPolicyFactory::calcExceptions( $this->getUserDateID() );
			} else {
				//Deleted is true. Just re-calc exceptions
				ExceptionPolicyFactory::calcExceptions( $this->getUserDateID() );
			}
		}

		/*
		if ( $this->getDeleted() == TRUE ) {
			$plf = new PunchListFactory();
			$plf->getByPunchControlID( $this->getId() );
			if ( $plf->getRecordCount() > 0 ) {
				foreach( $plf as $p_obj ) {
					$p_obj->setDeleted(TRUE);
					$p_obj->Save();
				}
			}
		}
		*/

		return TRUE;
	}
}
?>
